function P = get_reference_alloc_highq(bits)
%
% P = get_reference_alloc_highq(bits)
%
% Gets the allocation matrix, i.e. the assignment of reference bits for
% each coefficient of the 8x8 px DCT spectrum.
% 
% Input Parameters:
%
%   - bits - the total desired number of reference bits per block
%
% Output:
% 
% P - the bits allocation matrix
%
% ------------------------------------------------------------------------
%
% Copyright (C) Paweł Korus
% All rights reserved.
%
% Contact: Paweł Korus, pkorus@agh.edu.pl
%
% The author is with the Department of Telecommunications of AGH University
% of Science and Technology, Kraków, Poland.
%
%----------------------------------------------------------------------
%
% Permission to use, copy, or modify this software and its documentation
% for educational and research purposes only and without fee is hereby
% granted, provided that this copyright notice and the original authors'
% names appear on all copies and supporting documentation. This program
% shall not be used, rewritten, or adapted as the basis of a commercial
% software or hardware product without first obtaining permission of the
% authors. The authors make no representations about the suitability of
% this software for any purpose. It is provided "as is" without express
% or implied warranty.
%
    P = zeros(8);
    p = [8 8 8 7 7 7 6 6
         8 8 7 7 7 6 6 6
         8 7 7 7 6 6 6 5
         7 7 7 6 6 6 5 5
         7 7 6 6 6 5 5 5
         7 6 6 6 5 5 5 5
         6 6 6 5 5 5 5 4
         6 6 5 5 5 5 4 4];
     
     indices = zigzag(8);
     ci = 1;
     for b = 1:bits
         if P(indices(ci)) >= p(indices(ci))
             ci = ci + 1;
         end
         P(indices(ci)) = P(indices(ci)) + 1;
     end

end