%% Validate the designed descriptor - Random Rates

addpath('../include');
addpath('../include/alloc');
addpath('../include/fc');

clc;
trates = 0.3:0.025:0.5;

directory = '../testset/';
files = dir(strcat(directory,'*.png'));
L = size(files,1);
imgs = { files(:).name };
img_size = 256;

options = self_recovery_defaults();
options.SQ_LENGTH = 8;
options.qdf_resolution = 2;

load ../data/QProfiles_2LSB_4profiles_b88

%% Perform the test

M = 200;
successes = zeros(2,M);
failures = zeros(2,M);
si = 1;
fi = 1;

qd_mode = 0; % 0 - Pessimistic; 1 - Average

for i = 1:M
    
        % Select image on random
        k = 1 + randint(1,1,L);
        img = imread2(strcat(directory,files(k).name), img_size);
    
        % Select random tampering rates
        rand('twister', sum(100*clock));
        t_rate = 0.25 + 0.25*rand();
        eff_rate = 0.2 + 0.4*rand();
    
        % Prepare the descriptor
        [QD, QDf] = map_profiles(double(img/4), QProfiles);
        [QD_opt, at_rate] = qd_optimize(QD, QDf, zeros(size(QD)), t_rate, qd_mode, QProfiles.guard_levels, 16, QProfiles.lambdas);
        if round(100*at_rate) < round(100*t_rate)
            fprintf('ERROR Design failed - needed: %.2f / obtained: %.2f\n', t_rate, at_rate);
            return
        end
        options.QD = QD_opt;
        
        % Encode
        tic;
        [imgw, r] = c_encode(img, options, QProfiles);
        fprintf('Encoding - %.3f seconds\n', toc);
        
        % Perform tampering - if qd_mode = 1
        if qd_mode == 1
            imgt = tamper(imgw, eff_rate);
        else
            imgt = tamper_prior(imgw, eff_rate, r.QD); 
        end

        % Decode
        [imgr, rd] = c_decode(imgt, options, QProfiles);
        if rd.str_errors == 0
            successes(:,si) = [t_rate; eff_rate];
            si = si + 1;
        else
            failures(:,fi) = [t_rate; eff_rate];
            fi = fi + 1;
            if eff_rate < t_rate 
                fprintf('ERROR Fatal error - should be: %.2f / tampering: %.2f\n', at_rate, eff_rate);
            end
        end
end

successes(:,si:end) = [];
failures(:,fi:end) = [];

%% Plot Results

clf;
plot(successes(1,:), successes(2,:), 'go');
hold on;
plot(failures(1,:), failures(2,:), 'rx');
plot(0.25:0.001:0.5,0.25:0.001:0.5, 'k');
