%% 4 Successive Descriptors and Reconstruction Example 

% Show 4 successive descriptors

addpath('../include');
addpath('../include/alloc');
addpath('../include/fc');

clc;
qd_mode = 1;
trates = 0.30:0.05:0.5;

img = imread2('../testset/lena.png', 256);

options = self_recovery_defaults();

load ../data/QProfiles_2LSB_DA_b88

Imap = zeros(size(img)/8);

QDs = zeros(size(img,1)/8,size(img,2)/8, 5);
i = 1;
for t_rate = trates
    
    [QD, QDf] = map_profiles(double(img/4), QProfiles);
    [QD_opt, gamma_out] = qd_optimize(QD, QDf, Imap, t_rate, qd_mode, QProfiles.guard_levels, 16, QProfiles.lambdas);
    if gamma_out < t_rate
        fprintf('Design failed! Constraint not met: %.2f vs %.2f\n', t_rate, gamma_out);
    end
    QDs(:,:,i) = QD_opt;
    i = i + 1;
end

%% Plot the results

figure(1);
clf;
subplot(2,3,1);
imagesc(img);

map = linspace(0,1,length(QProfiles.lambdas));
map = repmat(map,3,1)';

for i = 1:5
    subplot(2,3,i+1);
    imshow(QDs(:,:,i), map, 'InitialMagnification', 'fit');
        
    [w,l] = qd2weights(QDs(:,:,i), QProfiles.lambdas);
    [gamma_1, gamma_2] = plot_demand(w(end:-1:1), l(end:-1:1), numel(QD), -1);
    
    title(strcat(sprintf('\\gamma = %.2f / %.2f', trates(i), gamma_2), ' | ', sprintf(' %d', unique(QDs(:,:,i))), ' | ', sprintf(' %d', l) ));
%     imwrite(QDs(:,:,i), map, sprintf('./tmp/map-%03d.png', 100*trates(i)));
end

%% Plot demands

figure(2);
clf;
subplot(2,3,1);
imagesc(img);
for i = 1:5
    subplot(2,3,i+1);
    [w,l] = qd2weights(QDs(:,:,i), QProfiles.lambdas);
    [gamma_1, gamma_2] = plot_demand(w(end:-1:1), l(end:-1:1), numel(QD), 1);    
    hold on;
    plot(trates(i)*[1 1], [trates(i), 1], 'g--')
    title(strcat(sprintf('\\gamma = %.2f / %.2f', trates(i), gamma_2), ' | ', sprintf(' %d', l), ' | ', sprintf(' %.0f', w) ));
end

%% Reconstruction Attemps

load ../data/QProfiles_2LSB_DA_b88

o = self_recovery_defaults();
o.LSBS = 2;
o.QD = QDs(:,:,size(QDs,3));
imgr = get_reference_high(img, o, QProfiles);
figure(4);
imagesc(imgr); title('With importance map');

%% Reconstruction Attemps

load ../data/QProfiles_2LSB_5profiles_b96

o = self_recovery_defaults();
o.LSBS = 2;
imgr = get_reference_ref(img, o, QProfiles, 2);
figure(5); clf;
imagesc(imgr); title('Reference scheme');