%% Compare Two Allocation Matrices
%
% Compare the fidelity of reference images for different allocation
% matrices.

addpath('../include/');
addpath('../include/alloc');
addpath('../include/bnb');

%% Setup the Experiment

testset = '../testset/';
    
mode = {'lloyd', 'uniform', 'lloyd-pi'};
dataset = {'sipi', 'bows', 'iceland', 'ucid'};

qi = 1;         % use lloyd quantizer
di = 2;         % use bows training set - for quantizers and dist. models

sa_mode = 0;    % use blocks for all texture levels
dr_factor = 8;  % L = 3 LSBs
use_ssim = 0;

b = 54;         % number of reference bits

% Referende matrix
Pref = [8 6 4 3 2 0 0 0
        6 4 3 2 0 0 0 0
        4 3 2 0 0 0 0 0
        3 2 0 0 0 0 0 0
        2 0 0 0 0 0 0 0
        0 0 0 0 0 0 0 0
        0 0 0 0 0 0 0 0
        0 0 0 0 0 0 0 0]; 

P0 = simple_alloc(b, 'exp', sprintf('../quantizers/DM_%d_%s_%d_%s.mat', sa_mode, dataset{di}, dr_factor, mode{qi}));

if sum(P0(:)) ~= sum(Pref(:))
    display('WARNING Allocation matrices have different total number of bits.');
    return;
end

%% Perform the tests

[psnr_t1, ssim_t1, ~, psnrs_t1, ssims_t1] = rank_matrix(Pref, dataset{di}, mode{qi}, 8, testset);
[psnr_t2, ssim_t2, imgs, psnrs_t2, ssims_t2] = rank_matrix(P0, dataset{di}, mode{qi}, 8, testset);

%%
clc;

if use_ssim == 0
    t1 = psnrs_t1; t2 = psnrs_t2; 
    st1 = psnr_t1; st2 = psnr_t2; 
else
    t1 = ssims_t1; t2 = ssims_t2; 
    st1 = ssim_t1; st2 = ssim_t2; 
end

fprintf('[%s]\n\n', testset);
fprintf('     \t\t Ref. \t\t Optimized \t Diff\t ');
fprintf('\n');

for i = 1:length(imgs)
    fprintf('%s \t ', imgs{i});
    fprintf('%.2f dB \t ', [t1(i) t2(i) (t2(i)-t1(i))]);
    fprintf('\\\\\n');
end

fprintf('[mean]\t\t ');
fprintf('%.2f dB \t ', [st1 st2]);
fprintf('\\\\\n');

fprintf('[median] \t ');
fprintf('%.2f dB \t ', median([t1 t2]));
fprintf('\\\\\n\n');

%%

clf;
subplot(2,2,1);
boxplot([t1 t2]); 
title('Box-plot');

subplot(2,2,2);
boxplot([remove_outliers(t1, 0.95) remove_outliers(t2, 0.95)])
title('Box-plot for 95% most typical scores');

subplot(2,1,2);
[pi1, xi1] = hist(remove_outliers(t1, 1), 15:1:40);
[pi2, xi2] = hist(remove_outliers(t2, 1), 15:1:40);
h = bar(xi1, pi1, 'hist');  set(h, 'facecolor', 'r'); hold on;
h = bar(xi2, pi2, 'hist');  set(h, 'facecolor', 'b');
hold off;
legend('Ref', 'Optimized');
h = findobj(gca,'Type','patch');
set(h,'facealpha',0.5);
title('Histograms');
