%% Content-Adaptive vs. Reference Scheme 
%
% Compares reference images for the content-adaptive and the reference
% schemes. 
addpath('../include');
addpath('../include/alloc');

options = self_recovery_defaults();
options.verbose = 0;
options.LSBS = 3;

profile_id = 1;

% List images in the test set
directory = '../testset/';
files = dir(strcat(directory,'*.png'));
L = size(files,1);
imgs = { files(:).name };

% Select image for the demonstration
img_id = 1;
img = imread2(strcat(directory,files(img_id).name));

%% Content-Adaptive Scheme

if options.LSBS == 3
    load ../data/QProfiles_3LSB_CA_b152_extended
elseif options.LSBS == 2
    load ../data/QProfiles_2LSB_CA_b88_extended
end

% Select the best profiles for all image blocks
options.QD = map_profiles(double(floor(double(img)/pow2(options.LSBS))), QProfiles, [1:4]+4*(profile_id-1));

% Print stats
P0 = QProfiles.P_profiles(:,:,1+4*(profile_id-1));
P1 = QProfiles.P_profiles(:,:,2+4*(profile_id-1));
P2 = QProfiles.P_profiles(:,:,3+4*(profile_id-1));
P3 = QProfiles.P_profiles(:,:,4+4*(profile_id-1));
fprintf('C-A scheme - using profiles: %d %d %d %d\n', sum(P0(:)), sum(P1(:)), sum(P2(:)), sum(P3(:)));

% Get the reconstruction reference
imgr = get_reference_high(img, options, QProfiles);
imwrite(imgr, '../output/reference-adaptive.png');

subplot(1,2,1);
imagesc(imgr); 
title(sprintf('CA: %.2f dB / %.2f\n', calc_psnr(img, imgr), ssim_index(img, imgr)));

%% Reference Scheme

if options.LSBS == 3
    load ../data/QProfiles_3LSB_4profiles_b160
elseif options.LSBS == 2
    load ../data/QProfiles_2LSB_5profiles_b96
end

% Get reference image for the traditional reference scheme
imgr = get_reference_ref(img, options, QProfiles, profile_id);

% Print stats
P4 = QProfiles.P_profiles(:,:,profile_id);
fprintf('Ref. scheme - using profile: %d\n', sum(P4(:)));

imwrite(imgr, '../output/reference-ref.png');

subplot(1,2,2);
imagesc(imgr); 
title(sprintf('REF: %.2f dB / %.2f\n', calc_psnr(img, imgr), ssim_index(img, imgr)));
linkaxes;

%% QD Stats

display('Profile usage statistics');
q_val = unique(options.QD(:));
for i = 1:length(q_val)
    eqmap = (options.QD(:) == q_val(i));
    fprintf('Profile %d -> %d / %d (%.2f%%)\n', i, sum(eqmap), numel(eqmap), mean2(eqmap)*100);
end