function [x0] = get_alloc_starting_point(beq, mode)
%
% [x0] = get_alloc_starting_point(beq, mode)
%
%   beq - number of bits to be allocated
%
%   mode - operation mode:
%          0 - uniform
%          1 - low-freq. first
%          2 - shaped
%          3 - <none>
%          4 - ?
%          5 - uniform, 1 is disallowed
%
    x0 = zeros(15,1);
    Mx  = [8 7 7 6 6 6 5 5 5 5 4 4 4 4 4];
    Aeq = [1 2 3 4 5 6 7 8 7 6 5 4 3 2 1];
    if mode == 0
        
        % Approximate Uniform Allocation
        xi = 1;
        while Aeq*x0 < beq
            x0(xi) = x0(xi) + 1;
            xi = xi+1;
            if xi > 15
                xi = 1;
            end
%             fprintf('%d', x0);
%             fprintf(' : %d\n', Aeq*x0);
        end
        di = Aeq*x0 - beq;
        if di > 0
            xi = xi - 1;
            x0(xi) = x0(xi) - 1;
        end
        di = Aeq*x0 - beq;
        while di > 0
            di = min(min(di, 9), find(x0, 1, 'last'));
            x0(di) = x0(di) - 1;
            di = Aeq*x0 - beq;
        end
        while di < 0
            di = max(min(-di, 9), find(x0, 1, 'first'));
            x0(di) = x0(di) + 1;
            di = Aeq*x0 - beq;
        end

%         fprintf('%d', x0);
%         fprintf(' : %d\n', Aeq*x0);
        
    elseif mode == 1
        
        % Approximate Uniform Allocation
        xi = 1;
        while Aeq*x0 < beq
            x0(xi) = x0(xi) + 1;
            if x0(xi) == 8
                xi = xi+1;
                if xi > 15
                    xi = 1;
                end
            end
%             fprintf('%d', x0);
%             fprintf(' : %d\n', Aeq*x0);
        end
        di = Aeq*x0 - beq;
        while di > 0
            di = min(min(di, 9), find(x0, 1, 'last'));
            x0(di) = x0(di) - 1;
            di = Aeq*x0 - beq;
        end
%         fprintf('%d', x0);
%         fprintf(' : %d\n', Aeq*x0);
        
    elseif mode == 2
        
        % Approximate Uniform Allocation
        xi = 1;
        while Aeq*x0 < beq
            x0(xi) = x0(xi) + 1;
            if x0(xi) == Mx(xi)
                xi = xi+1;
                if xi > 15
                    xi = 1;
                end
            end
%             fprintf('%d', x0);
%             fprintf(' : %d\n', Aeq*x0);
        end
        di = Aeq*x0 - beq;
        while di > 0
            di = min(min(di, 9), find(x0, 1, 'last'));
            x0(di) = x0(di) - 1;
            di = Aeq*x0 - beq;
        end
%         fprintf('%d', x0);
%         fprintf(' : %d\n', Aeq*x0);
        
    elseif mode == 4
        
        % Approximate Uniform Allocation
        xi = 0;
        while Aeq*x0 <= beq
			xi = mod(xi,15)+1;
            x0(xi) = x0(xi) + 1;
        end
        x0(xi) = x0(xi) - 1;
        while Aeq*x0 - beq < 0
            di = beq - Aeq*x0;
            x0(di) = x0(di) + 1;
        end

%         fprintf('%d', x0);
%         fprintf(' : %d\n', Aeq*x0);

    elseif mode == 5
        
        % Approximate Uniform Allocation
        xi = 0;
        while Aeq*x0 <= beq
            xi = mod(xi,15) + 1;
            if x0(xi) == 0
                x0(xi) = x0(xi) + 2;
            else
                x0(xi) = x0(xi) + 1;    
            end
        end
        if x0(xi) == 2
            x0(xi) = x0(xi) - 2;
        else
            x0(xi) = x0(xi) - 1;
        end
        
        xi = 0;
        while Aeq*x0 <= beq
            xi = mod(xi,15) + 1;
            if x0(xi) == 0
                x0(xi) = x0(xi) + 2;
            else
                x0(xi) = x0(xi) + 1;    
            end
        end
        if x0(xi) == 2
            x0(xi) = x0(xi) - 2;
        else
            x0(xi) = x0(xi) - 1;
        end
        
        while Aeq*x0 - beq < 0
            di = beq - Aeq*x0;
            x0(di) = x0(di) + 1;
        end

%         fprintf('%d', x0);
%         fprintf(' : %d\n', Aeq*x0);

	end


    
    
end