function [P, xo, d] = simple_alloc(beq, model, model_file, min_valid_prec)
% [P, xo, d] = simple_alloc(beq, model, model_file, min_valid_prec)
%
% Vector-like optimization
%
% model   = 'lin' or 'exp'
% model_file = e.g., './quantizers/DM_0_bows_2_lloyd.mat'
% min_valid_prec - optional, default = 2

if (nargin < 4) || (min_valid_prec < 1) || (min_valid_prec > 8)
    min_valid_prec = 2;
end

P = zeros(8,8);
IM = zeros(8,8);
indices = zigzag(8);
    
% Update distortion assessment
% Initialize distortion models
if all(strcmp(model, 'lin'))
    fun = @(x) dm_model_mlin(x, model_file);
else
    fun = @(x) dm_model_mexp(x, model_file);
end 

xf = zeros(64,1);
impacts_benef = nan(64,1);

% calcualte the gain in the objective function due to promotion of
% individual components - when increasing from 0 - divide by 2 - because
% promotion needs to skip 1
for oi = 1:64
   xt = xf;
   xt(oi) = xt(oi) + min_valid_prec;
   impacts_benef(oi) = (fun(xf) - fun(xt))/min_valid_prec;
end

while sum(xf) < beq       
   % Find the best beneficiary
   [~, bi] = max(impacts_benef);
   
   if xf(bi) == 0 && sum(xf) >= beq - min_valid_prec + 1
       impacts_benef(bi) = NaN;
       continue;
   end
   
   if isnan(impacts_benef(bi))
       fprintf('ERROR Constraint cannot be met! Only %d bits allocated\n', sum(xf));
       break;
   end
   if xf(bi) == 0
       xf(bi) = xf(bi) + min_valid_prec;
   else
       xf(bi) = xf(bi) + 1;
   end
   if xf(bi) == 8
       impacts_benef(bi) = NaN;
   else
       xt = xf;
       xt(bi) = xt(bi) + 1;
        impacts_benef(bi) = fun(xf) - fun(xt);
   end
end

if sum(xf) ~= beq
    fprintf('ERROR constraint not met! %d instead of %d', sum(xf), beq);
end

P(indices(:)) = xf;
d = fun(xf);
xo = xf;

