function [bits_out] = c_rlf_self_decode(bits_in, bits_ref, block, error_map, reconstruction_map, key)
%
% bits_out = c_rlf_self_decode(bits_in, bits_ref, block, error_map, 
%                       reconstruction_map, key)
%
% Implementation of the self-recovery decoder. Simplifies the content
% reconstruction problem by removing the dependencies on known symbols and
% by decoding only the necessary ones. After simplification, uses Gaussian
% elimination for solving the remaining problem.
%
% This version is a wrapper for the C++ RLF code written by Jarosław Białas.
% 
% Input Parameters:
%
%   - bits_in - input bitstream, vector of 0 and 1 (uint16)
%
%   - bits_ref - input bitstream with valid symbols regenerated from
%                     the remaining authentic image fragments, same length
%                     as 'bits_in' (uint16)
%
%   - block   - symbol length
%
%   - error_map - the tampering map, vector of 0 and 1, 
%                 tampered as 0-s (uint8)
%
%   - reconstruction_map - vector of 0 and 1, marks the original symbols
%                          which actually need to be decoded
%                          use 0-s for symbols that need decoding (uint8)
%
%   - key - integer, the seed for the PRNG
%
% Output:
% 
% bits_out - decoded bitstream, vector of 0 and 1
%
% ------------------------------------------------------------------------
%
% Copyright (C) Paweł Korus
% All rights reserved.
%
% Version: 2012.04.20
% Contact: Paweł Korus, pkorus@agh.edu.pl
%
% The author is with the Department of Telecommunications of AGH University
% of Science and Technology, Kraków, Poland.
%
% ------------------------------------------------------------------------
%
% Permission to use, copy, or modify this software and its documentation
% for educational and research purposes only and without fee is hereby
% granted, provided that this copyright notice and the original authors'
% names appear on all copies and supporting documentation. This program
% shall not be used, rewritten, or adapted as the basis of a commercial
% software or hardware product without first obtaining permission of the
% authors. The authors make no representations about the suitability of
% this software for any purpose. It is provided "as is" without express
% or implied warranty.
%----------------------------------------------------------------------
%
% The research leading to these results has received funding from the
% INDECT project funded by European Community's Seventh Framework Pro-
% -gramme (FP7 / 2007-2013) under grant agreement no.[218086].
%  