function [imgs, psnrs, ssims] = multi_image_test(P, dataset, quantizer, dr_factor, directory)
%
% [imgs, psnrs, ssims] = multi_image_test(test_name, 
%                                         P, 
%                                         dataset,
%                                         quantizer, 
%                                         dr_factor, 
%                                         directory)
%
% Simulates rec. reference generation for a set of test images.
%
% Params:
%
%
%  - P         - resource allocation matrix, 8x8
%
%  - dataset   - training set used for generating quant. code-books:
%                'bows', 'sipi', 'ucid', ...
%
%  - quantizer - quantization code-book name,
%                'lloyd', 'lloyd-pi', 'uniform'
%
%  - dr_factor - dynamic range division factor, for LSB discarding 
%                (e.g., 8 for 3 LSBs)
%
%  - directory - path to test set
%

if ~exist('directory' ,'var')
    directory = '../testset/';
end
files = dir(strcat(directory,'*.png'));

L = size(files,1);

psnrs = zeros(L,1);
ssims = zeros(L,1);
imgs = { files(:).name };

for k = 1:L 
    fprintf('%3d of %3d\n', k, L);
    img = imread2(strcat(directory,files(k).name));
    [psnr, ssim, imgc] = single_image_test(img, P, dataset, quantizer, dr_factor);
    psnrs(k) = psnr;
    ssims(k) = ssim;
end

end