function QD = plant_qd(qds, weights, profiles, randomize_qd)
%
% QD = plant_qd(qds, weights, profiles, randomize_qd)
%
% Generates an artificial quality descriptor with given weights (frequencies)
% of individual quality profiles. If not randomized, the profiles are assigned
% sequentially: 
%
%   1		H+1		2H+1		...
%   2		H+2		2H+2		...
%   3		H+3		2H+3		...
%	...		...		...			...
%
% Parameters:
%
%   - qds			- QD size vector, e.g., [32, 32]
%
%   - weights		- profiles' weights
%
%   - profiles		- the list of profiles to be planted
%
%   - randomize_qd	- self-explanatory, optional
%
% Output:
%
%   - QD			- the resulting quality descriptor
%

    N = prod(qds);
    weights = weights/sum(weights);
    counts = round(weights*N);
    counts(end) = counts(end) - (sum(counts) - N);

    qd = zeros(1,N);
    qi = 1;
    for i = 1:length(profiles);
        qd(qi:qi+counts(i)-1) = profiles(i);
        qi = qi + counts(i);
    end
    if nargin == 4 && randomize_qd == true
        qd = qd(randperm(N));
    end
    QD = reshape(qd, qds);

end