function [w, l] = qd2weights(QD, full_lambdas)
%
% [w, l] = qd2weights(QD, full_lambdas)
%
% Provides statistics for a given quality descriptor.
% 
% Parameters:
% 
%   - QD			- quality descriptor
%
%   - full_lambdas	- vector with all lambda values in the given set of quality
%                     profiles
%
% Output:
% 
%   - w				- the weights
%
%   - l				- the actual lambdas in use
%

    lambdas = zeros(1,length(unique(QD(:))));
    li = 1;
    for k = unique(QD(:))'
        lambdas(li) = full_lambdas(k);
        li = li + 1;
    end
    
    if length(lambdas) > 1
        weights = hist(QD(:),unique(QD(:)));
    else
        weights = numel(QD);
    end
    wi = 1;
    for ul = unique(lambdas)
        l(wi) = ul;
        w(wi) = sum(weights(lambdas == ul));
        wi = wi + 1;
    end
end
