function [B, I] = quantize_block(B, P, quantizer, LM_quanta)
    I = zeros(size(B));
    if quantizer == 0
        B = round(B);
        % Matrix X is the maxmum absolute value that can be encoded for
        % each coefficient, as defined by P
        M = pow2(P-1)-1;
        M(1,1) = pow2(P(1,1))-1;

        % Saturate the values that exceed M
        B(B > M) = M(B > M);
        B(B < -M) = -(M(B < -M));

        % Zero the coefficients that have no anticipated bits
        B(P == 0) = 0;
    else
        Q_map = [0 1 2 3 4 5 6 7;
                 1 2 3 4 5 6 7 8;
                 2 3 4 5 6 7 8 9;
                 3 4 5 6 7 8 9 10;
                 4 5 6 7 8 9 10 11;
                 5 6 7 8 9 10 11 12;
                 6 7 8 9 10 11 12 13;
                 7 8 9 10 11 12 13 14];
        
        for qi = 1:64
            if P(qi) > 0
                codebook = LM_quanta{Q_map(qi)+1,P(qi)+1};
                [q, i] = quantize_scalar(B(qi), codebook);
                B(qi) = q;
                I(qi) = i;
            else
                B(qi) = 0;
                I(qi) = -Inf;
            end
        end
    end
end