function [sx, sy, fx, fy] = simulate_demand(weights, lambdas, image_size, count, randomize_qd, randomize_tamper)

    load data/QProfiles11
    img = imread2('../testset/lena.png', image_size);

    options = self_recovery_defaults();
    options.verbose = false;
    
    % Map lambdas to profiles
    available_lambdas = zeros(1, size(QProfiles.P_profiles,3));
    S_LENGTH = options.BLOCK*options.BLOCK*options.LSBS-options.H_LENGTH - options.SQ_LENGTH;
    for i = 1:length(available_lambdas)
        P = QProfiles.P_profiles(:,:,i);
        available_lambdas(i) = sum(sum(P))/S_LENGTH;        
    end
    profiles = zeros(size(lambdas));
    for i = 1:length(profiles)
        candidates = find(available_lambdas == lambdas(i), 1, 'last');
        if isempty(candidates) 
            throw(MException('error:Error', sprintf('Candidate profile not found for lambda %.2f', lambdas(i))));
        end
        profiles(i) = candidates;
    end
    % Generate random QD
    options.QD = plant_qd(size(img)/options.BLOCK, weights, profiles, randomize_qd);
    
    sy = zeros(1,count);
    sx = zeros(1,count);
    fy = zeros(1,count);
    fx = zeros(1,count);
    si = 1;
    fi = 1;
    
    for i = 1:count

        options.KEY = round(100*mean(clock()));
        t_rate = 0.7*rand();
        [imgw, r] = c_encode(img, options, QProfiles);
        
        if randomize_tamper == true
            imgt = tamper(imgw, t_rate);
        else
            imgt = tamper_prior(imgw, t_rate, r.QD);
        end
        
        [~, rd] = c_decode(imgt, options, QProfiles);

        if rd.str_errors == 0;
            sx(si) = t_rate;
            sy(si) = rd.reconstruction_demand;
            si = si + 1;
        else
            fx(fi) = t_rate;
            fy(fi) = rd.reconstruction_demand;
            fi = fi + 1;
        end
    end
    
    sy(si:end) = [];
    sx(si:end) = [];
    fy(fi:end) = [];
    fx(fi:end) = [];
end