function [psnr, ssim, imgc, qmap] = single_image_test(img, P, dataset, quantizer, dr_factor)
%
% [psnr, ssim, imgc] = single_image_test(img, P, dataset, quantizer, dr_factor)
%
% Simulates rec. reference generation for a given image with given settings.
%
% Params:
%
%  - img       - input image
%
%  - P         - resource allocation matrix, 8x8
%
%  - dataset   - training set used for generating quant. code-books:
%                'bows', 'sipi', 'ucid', ...
%
%  - quantizer - quantization code-book name,
%                'lloyd', 'lloyd-pi', 'uniform'
%
%  - dr_factor - dynamic range division factor, for LSB discarding 
%                (e.g., 8 for 3 LSBs)
%

    block = 8;

    Q = ones(8, 8);
    v1 = 5;
    v2 = 20;

    T = dctmtx(8);
    f = @(x) (T*x*T')./Q;
    fi = @(x) T'*(x.*Q)*T;
    
    variable_p_mode = false;
    if size(P,3) == 3
        variable_p_mode = true;
        LM_quantas = cell(1,3);
        for sa_mode = 1:3
            load(sprintf('../quantizers/CB_%d_%s_%d_%s.mat', sa_mode, dataset, dr_factor, quantizer));
            LM_quantas{sa_mode} = LM_quanta;
        end
    else
        load(sprintf('../quantizers/CB_%d_%s_%d_%s.mat', 0, dataset, dr_factor, quantizer));
    end

    imgo = img;
    img = double(floor(double(img)/dr_factor));
    S = blkproc(img, [8,8], f);
    [H, W] = size(img);
    qmap = zeros(H/block,W/block);
    for bx = 1:W/block
        for by = 1:H/block
            B = S(1+(by-1)*block:(by)*block, 1+(bx-1)*block:(bx)*block);
            if variable_p_mode == false
                B = quantize_block(B, P, 1, LM_quanta);
            else
                I = img(1+(by-1)*block:(by)*block, 1+(bx-1)*block:(bx)*block);
                sa = std(double(I(:)));
                B1 = quantize_block(B, P(:,:,1), 1, LM_quantas{1});
                B2 = quantize_block(B, P(:,:,2), 1, LM_quantas{2});
                B3 = quantize_block(B, P(:,:,3), 1, LM_quantas{3});
                d = [mean2(abs(I-fi(B1))) mean2(abs(I-fi(B2))) mean2(abs(I-fi(B3)))];
                [~, ind] = min(d);
                if ind == 1
                    B = B1;
                    qmap(by,bx) = 1;
                elseif ind == 2
                    B = B2;
                    qmap(by,bx) = 2;
                else
                    B = B3;
                    qmap(by,bx) = 3;
                end
                
            end
            S(1+(by-1)*block:(by)*block, 1+(bx-1)*block:(bx)*block) = B;
        end
    end

    imgc = blkproc(S, [8,8], fi);
    imgc = uint8(round(dr_factor*imgc+pow2(-1+log2(dr_factor))*ones(size(img))));

    psnr = calc_psnr(uint8(imgc), uint8(imgo));
    ssim = ssim_index(uint8(imgc), uint8(imgo));

end