function imgt = tamper(imgw, t, tampering_source, tampering_pattern, BLOCK)

    [H, W] = size(imgw);
    if ~exist('BLOCK', 'var'), BLOCK = 8; end

    % TAMPER SOME BLOCKS
    imgt = imgw;

    % Select random image frgmants that will be replaced with content from
    % another image
    
    % Determine the number of tampered blocks from the specified tampering
    % rate
    T = int32(W*H/BLOCK/BLOCK*t);
    r.gamma = double(T) / (W*H/BLOCK/BLOCK);
    
    % Generate a tampered image as a source for content tampering
    
    if ~exist('tampering_source', 'var')
        tampering_source = 'random';
    end
    
    if ~exist('tampering_pattern', 'var')
        tampering_pattern = 'random';
    end

    
    if strcmp(tampering_source, 'random')
        imgs = imresize(imgw', size(imgw));
        imgs = imgs + uint8(randint(size(imgw,1),size(imgw,2),32));
    else            
        if ~exist(tampering_source, 'file')
            fprintf('ERROR File does not exist!\n');
            return;                
        end
            
        imgs = imread(tampering_source);
        
        if size(imgs,3) > 1
            imgs = uint8(mean(imgs,3));
        end

        imgs = imresize(imgs, size(imgw));
    end
    
    if strcmp(tampering_pattern, 'random')
        
        fprintf('Using random tampering pattern with rate %.4f\n', r.gamma);
        ts = randperm(W*H/BLOCK/BLOCK);
        ts = ts(1:T);
        
    elseif strcmp(tampering_pattern, 'continuous')
        
        fprintf('Using continuous tampering pattern with rate %.4f\n', r.gamma);
        ts = 1:T;
    else
        fprintf('WARNING Unknown tampering pattern\n');
        return;
    end
    
    % Actual tampering
    for bx = 1:W/BLOCK
        for by = 1:H/BLOCK
            block_id = by + (bx-1)*H/BLOCK - 1 + 1;
            if numel(find(ts == block_id)) > 0
                B = imgs(1+(by-1)*BLOCK:(by)*BLOCK, 1+(bx-1)*BLOCK:(bx)*BLOCK);
                imgt(1+(by-1)*BLOCK:(by)*BLOCK, 1+(bx-1)*BLOCK:(bx)*BLOCK) = B;
            end
        end
    end

end