function bits_out = RLF_encode(bits_in, block, blocks_needed, key)
%
% bits_out = RLF_encode(bits_in, block, blocks_needed, key)
%
% Implementation of the random linear fountain encoder. 
% 
% Input Parameters:
%
%   - bits_in - input bitstream, vector of 0 and 1
%
%   - block   - symbol length
%
%   - blocks_needed  - number of desired symbols of the encoded stream
%
%   - key - integer, the seed for the PRNG
%
% Output:
% 
% bits_out - encoded stream, vector of 0 and 1
%
% ------------------------------------------------------------------------
%
% Copyright (C) 2011 Paweł Korus <pkorus@agh.edu.pl>
%
% Permission to use, copy, or modify this software and its documentation
% for educational and research purposes only and without fee is hereby
% granted, provided that this copyright notice and the original authors'
% names appear on all copies and supporting documentation. This program
% shall not be used, rewritten, or adapted as the basis of a commercial
% software or hardware product without first obtaining permission of the
% authors. The authors make no representations about the suitability of
% this software for any purpose. It is provided "as is" without express
% or implied warranty.
%


    % Pad with 0's
    if mod(length(bits_in), block) ~= 0
        pad = zeros(1, block - mod(length(bits_in), block));
        bits_in = [bits_in pad];
    end
    
    % Create a matrix for the resulting bit-stream
    bits_out = zeros(1, block*blocks_needed, 'uint16');
    
    % Initialize the PRNG
    rand('twister', key);
    
    for i = 1:blocks_needed
        ro = randint(1, length(bits_in)/block, 2); 
        P = find(ro);
        B = zeros(numel(P), block);
        for b = 1:numel(P)
            bi = P(b);
            B(b, :) = bits_in(1+(bi-1)*block:bi*block);
        end
        if numel(P) == 1
            bits_out(1+(i-1)*block:i*block) = mod(B, 2);
        else
            bits_out(1+(i-1)*block:i*block) = mod(sum(B), 2);
        end
    end
end
